class ClientAuthentication
  attr_accessor :header_keys

  # model must respond to :key and :secret
  def initialize(headers, max_seconds = 5, model = Application, logger=nil)
    @headers = headers
    @max_seconds = max_seconds
    @model = model
    @logger = logger

    @header_keys = {
      time:   "X-Level3-Digest-Time",
      key:    "X-Level3-Application-Key",
      digest: "X-Level3-Digest",
    }
  end

  def authenticate_client!
    key =    @headers[ header_keys[:key] ]
    time =   @headers[ header_keys[:time] ]
    digest = @headers[ header_keys[:digest] ]

    authenticate_header_values!(key, digest, time)
    key
  end

  def authenticate_header_values!(key, digest, time)
    diff = (time.to_i - salt.to_i).abs
    raise ClientAuthenticationException.new if diff > @max_seconds

    application = @model.find_by_key(key)
    raise ClientAuthenticationException.new unless application

    secret = application.secret
    raise ClientAuthenticationException.new unless secret

    raise ClientAuthenticationException.new unless digest.eql? generate_digest(time, secret)
    true
  end

  def generate_digest(salt, secret)
    Base64.encode64(
      OpenSSL::HMAC.digest(
        'sha256',secret, salt) ).
      strip
  end

  def salt
    Time.now.to_i.to_s
  end

  class ClientAuthenticationException < Exception
  end

end
